import { Platform, TouchableOpacity, View, Image } from 'react-native';
import React, { FC, useEffect, useRef } from 'react';
import MapView, { UrlTile } from 'react-native-maps';

import { styles } from './styles';
import { API_HOST, FASTEST_MAP_HOST } from 'src/constants';
import { NavigationProp } from '@react-navigation/native';
import { AvatarWithInitials } from 'src/components';
import { Colors } from 'src/theme';

type Props = {
  navigation: NavigationProp<any>;
  route: any;
};

const UsersMapScreen: FC<Props> = ({ navigation, route }) => {
  const userId = route.params?.userId;
  const data = route.params?.data;

  const tilesBaseURL = `${FASTEST_MAP_HOST}/tiles_osm`;
  const gridUrl = `${FASTEST_MAP_HOST}/tiles_nm/grid`;
  const visitedTiles = `${FASTEST_MAP_HOST}/tiles_nm/user_visited/${userId}`;

  const mapRef = useRef<MapView>(null);

  useEffect(() => {
    navigation.getParent()?.setOptions({
      tabBarStyle: {
        display: 'none',
        ...Platform.select({
          android: {
            height: 58
          }
        })
      }
    });
  }, [navigation]);

  const renderMapTiles = (url: string, zIndex: number, opacity = 1) => (
    <UrlTile
      urlTemplate={`${url}/{z}/{x}/{y}`}
      maximumZ={15}
      maximumNativeZ={13}
      shouldReplaceMapContent
      minimumZ={0}
      opacity={opacity}
      zIndex={zIndex}
    />
  );

  const handleGoBack = () => {
    navigation.getParent()?.setOptions({
      tabBarStyle: {
        display: 'flex',
        ...Platform.select({
          android: {
            height: 58
          }
        })
      }
    });
    navigation.goBack();
  }

  return (
    <View style={styles.container}>
      <MapView
        initialRegion={{
          latitude: 0,
          longitude: 0,
          latitudeDelta: 180,
          longitudeDelta: 180
        }}
        ref={mapRef}
        showsMyLocationButton={false}
        showsCompass={false}
        zoomControlEnabled={false}
        style={styles.map}
        mapType={Platform.OS == 'android' ? 'none' : 'standard'}
        maxZoomLevel={15}
        minZoomLevel={0}
      >
        {renderMapTiles(tilesBaseURL, 1)}
        {renderMapTiles(gridUrl, 2)}
        {userId && renderMapTiles(visitedTiles, 2, 0.5)}
      </MapView>

      <TouchableOpacity
        style={[styles.cornerButton, styles.topLeftButton]}
        onPress={handleGoBack}
      >
        {data.avatar ? (
          <Image
            style={{
              borderRadius: 48 / 2,
              width: 48,
              height: 48,
              borderWidth: 2,
              borderColor: Colors.WHITE
            }}
            source={{ uri: API_HOST + data.avatar }}
          />
        ) : (
          <AvatarWithInitials
            text={`${data.first_name[0] ?? ''}${data.last_name[0] ?? ''}`}
            flag={API_HOST + data.homebase_flag}
            size={48}
            borderColor={Colors.WHITE}
          />
        )}
      </TouchableOpacity>
    </View>
  );
};

export default UsersMapScreen;
